/******************************************************************************
 * QUANTA - A toolkit for High Performance Data Sharing
 * Copyright (C) 2003 Electronic Visualization Laboratory,  
 * University of Illinois at Chicago
 *
 * This library is free software; you can redistribute it and/or modify it 
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either Version 2.1 of the License, or 
 * (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public 
 * License for more details.
 * 
 * You should have received a copy of the GNU Lesser Public License along
 * with this library; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Direct questions, comments etc about Quanta to cavern@evl.uic.edu
 *****************************************************************************/

#include "QUANTAts_mutex_c.hxx"
#include "QUANTAmisc_debug.hxx""
#include <string.h>

QUANTAts_mutex_c :: QUANTAts_mutex_c()
{
#ifdef QUANTA_USE_PTHREADS
	pthread_mutex_init(&mutex,NULL);
#endif
#ifdef QUANTA_USE_WINDOWSTHREADS
    mutex = CreateMutex(NULL, FALSE, NULL);
#endif
	debugMsg = NULL;
	debugStat = QUANTAts_mutex_c::DEBUG_OFF;
}

void QUANTAts_mutex_c :: lock()
{
	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"BEFORE LOCK");
	}

#ifdef QUANTA_USE_PTHREADS
	pthread_mutex_lock(&mutex);
#endif
#ifdef QUANTA_USE_WINDOWSTHREADS
    WaitForSingleObject(mutex, INFINITE);
#endif

	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"AFTER LOCK");
	}
}

void QUANTAts_mutex_c :: unlock()
{
	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"BEFORE UNLOCK");
	}
#ifdef QUANTA_USE_PTHREADS
	pthread_mutex_unlock(&mutex);
#endif
#ifdef QUANTA_USE_WINDOWSTHREADS
    ReleaseMutex(mutex);
#endif

	if (debugStat == DEBUG_ON) {
		cvrnMesg(debugMsg,"AFTER UNLOCK");
	}
}

void QUANTAts_mutex_c :: setMutexDebug(QUANTAts_mutex_c::mutexDebug_t stat)
{
	debugStat = stat;
}

void QUANTAts_mutex_c :: setMutexDebugMesg(char *msg)
{
	if (debugMsg)
	  delete[] debugMsg;
	debugMsg = new char[strlen(msg)+1];
	strcpy(debugMsg,msg);
}

QUANTAts_mutex_c :: ~QUANTAts_mutex_c()
{
	if (debugMsg)
	  delete[] debugMsg;
#ifdef QUANTA_USE_PTHREADS
	pthread_mutex_destroy(&mutex);
#endif
#ifdef QUANTA_USE_WINDOWSTHREADS
    if (mutex)
      CloseHandle(mutex);
#endif
}

QUANTA_MUTEX_T * QUANTAts_mutex_c :: getMutex()
{
	return &mutex;
}

